<%@ Page language="C#" %>
<%@ Import Namespace="System.CodeDom" %>
<%@ Import Namespace="System.CodeDom.Compiler" %>
<%@ Import Namespace="System.Reflection" %>
<%@ Import Namespace="System.IO" %>
<%@ Import Namespace="Microsoft.CSharp" %>
<%@ Import Namespace="Microsoft.VisualBasic" %>
<html>
	<head>
		<script language="C#" runat="server">
    public string CreateCsCode(string filepath)
    {
      StreamWriter sw = null;
      try
      {
        sw = File.CreateText(filepath);

        CSharpCodeProvider codeProvider = new CSharpCodeProvider();
        ICodeGenerator codeGenerator = codeProvider.CreateGenerator(sw);
        CodeGeneratorOptions options = new CodeGeneratorOptions();
        options.BracingStyle = "C";

        //Create namespace references
        CodeSnippetCompileUnit usingSnippet1 = new CodeSnippetCompileUnit("using System;");
        codeGenerator.GenerateCodeFromCompileUnit(usingSnippet1, sw, options);
        sw.WriteLine();

        //Create namespace
        CodeNamespace recipesNamespace = new CodeNamespace("Recipes");

        //Create RoundMeasures Class Declaration
        CodeTypeDeclaration roundMeasures = new CodeTypeDeclaration();
        roundMeasures.IsClass = true;
        roundMeasures.Name = "RoundMeasures";
        roundMeasures.TypeAttributes = TypeAttributes.Public;

        //Create Round Class Declaration
        CodeTypeDeclaration round = new CodeTypeDeclaration();
        CodeComment roundComment = new CodeComment();
        roundComment.Text = "Round calculates the Area of a Circle, the Area of a Sphere, and the Volume of a Sphere";
        CodeCommentStatement roundCommentStatement = new CodeCommentStatement(roundComment);
        round.Comments.Add(roundCommentStatement);
        round.IsClass = true;
        round.Name = "Round";
        round.TypeAttributes = TypeAttributes.Public;

        //Create Class Member Field radius
        CodeMemberField radiusMember = new CodeMemberField(typeof(double),"radius");
        radiusMember.Attributes = MemberAttributes.Private;
        radiusMember.Comments.Add(new CodeCommentStatement("The radius used to calculate areas and volume."));
        round.Members.Add(radiusMember);

        //Create Round Class Constructor
        CodeConstructor roundConstructor = new CodeConstructor();
        roundConstructor.Attributes = MemberAttributes.Public;
        CodeParameterDeclarationExpression roundConParameter = new CodeParameterDeclarationExpression(typeof(double), "pRadius");
        roundConstructor.Parameters.Add(roundConParameter);
        roundConstructor.Statements.Add(new CodeSnippetStatement("this.radius = pRadius;"));
        round.Members.Add(roundConstructor);

        //Create Radius Property
        CodeMemberProperty radiusProperty = new CodeMemberProperty();
        radiusProperty.Attributes = MemberAttributes.Public;
        radiusProperty.Type = new CodeTypeReference(typeof(double));
        radiusProperty.Name = "Radius";
        radiusProperty.HasGet = true;
        radiusProperty.GetStatements.Add(new CodeSnippetExpression("return radius"));
        radiusProperty.HasSet = true;
        radiusProperty.SetStatements.Add(new CodeSnippetExpression("radius = value"));
        round.Members.Add(radiusProperty);

        //Create CircleArea Method
        CodeMemberMethod circleAreaMethod = new CodeMemberMethod();
        circleAreaMethod.Comments.Add(new CodeCommentStatement("Calculates the area of a circle."));
        circleAreaMethod.Name = "CircleArea";
        circleAreaMethod.ReturnType = new CodeTypeReference(typeof(double));
        circleAreaMethod.Attributes = MemberAttributes.Public;
        circleAreaMethod.Statements.Add(new CodeSnippetStatement("return Math.PI * radius * radius;"));
        round.Members.Add(circleAreaMethod);

        //Create SphereArea Method
        CodeMemberMethod sphereAreaMethod = new CodeMemberMethod();
        sphereAreaMethod.Comments.Add(new CodeCommentStatement("Calculates the area of a sphere."));
        sphereAreaMethod.Name = "SphereArea";
        sphereAreaMethod.ReturnType = new CodeTypeReference(typeof(double));
        sphereAreaMethod.Attributes = MemberAttributes.Public;
        sphereAreaMethod.Statements.Add(new CodeSnippetStatement("return 4 * CircleArea();"));
        round.Members.Add(sphereAreaMethod);

        //Create SphereVolume Method
        CodeMemberMethod sphereVolumeMethod = new CodeMemberMethod();
        sphereVolumeMethod.Comments.Add(new CodeCommentStatement("Calculates the volume of a sphere."));
        sphereVolumeMethod.Name = "SphereVolume";
        sphereVolumeMethod.ReturnType = new CodeTypeReference(typeof(double));
        sphereVolumeMethod.Attributes = MemberAttributes.Public;
        sphereVolumeMethod.Statements.Add(new CodeSnippetStatement("return SphereArea() * radius / 3;"));
        round.Members.Add(sphereVolumeMethod);

        //Add Round class to RoundMeasures class
        roundMeasures.Members.Add(round);

        //Create Main Method
        CodeMemberMethod mainMethod = new CodeMemberMethod();
        mainMethod.Name = "Main";
        mainMethod.Attributes = MemberAttributes.Public;
        mainMethod.Attributes = MemberAttributes.Static;
        mainMethod.Statements.Add(new CodeSnippetStatement("double r = 5.0;"));
        mainMethod.Statements.Add(new CodeSnippetStatement("Round rnd = new Round(r);"));
        mainMethod.Statements.Add(new CodeSnippetStatement("Console.WriteLine(\"Radius of Circle: {0:f1}\", rnd.Radius);"));
        mainMethod.Statements.Add(new CodeSnippetStatement("Console.WriteLine(\"Area of Circle: {0:f1}\", rnd.CircleArea());"));
        mainMethod.Statements.Add(new CodeSnippetStatement("Console.WriteLine(\"Area of Sphere: {0:f1}\", rnd.SphereArea());"));
        mainMethod.Statements.Add(new CodeSnippetStatement("Console.WriteLine(\"Volume of Sphere: {0:f1}\", rnd.SphereVolume());"));
        roundMeasures.Members.Add(mainMethod);

        //Add RoundMeasures class to Recipe namespace
        recipesNamespace.Types.Add(roundMeasures);

        //Generate Source Code File
        codeGenerator.GenerateCodeFromNamespace(recipesNamespace, sw, options);
        
        return filepath + " was successfully created.";
      }
      catch(Exception ex)
      {
        return "Error: " + ex.Message;
      }
      finally
      {
        //Close StreamWriter
        if (sw != null) sw.Close();
      }
    }

    private void btnGenerate_Click(object sender, System.EventArgs e)
    {
      Result.Text = CreateCsCode(Server.MapPath("Round.cs.txt"));
      link.Text = "View Code";
      link.NavigateUrl = "Round.cs.txt";
    }
		</script>
		<title>Generating a Class Using the CodeDOM</title>
	</head>
	<body>
		<form id="CreateCode" method="post" runat="server">
			<asp:Button id="btnGenerate" runat="server" Text="Create Code" OnClick="btnGenerate_Click"></asp:Button><br>
			<br>
			<asp:Label id="Result" runat="server"></asp:Label>
			<asp:HyperLink ID="link" Runat="server"/>
		</form>
	</body>
</html>
