<%@ Page language="C#" %>
<%@ Import Namespace="System.CodeDom" %>
<%@ Import Namespace="System.CodeDom.Compiler" %>
<%@ Import Namespace="System.Reflection" %>
<%@ Import Namespace="System.IO" %>
<%@ Import Namespace="System.Text" %>
<%@ Import Namespace="Microsoft.CSharp" %>
<%@ Import Namespace="Microsoft.VisualBasic" %>
<html>
 <head>
  <script language="C#" runat="server">
    public string CreateCsCode()
    {
      try
      {
        //Language specific code provider
        CSharpCodeProvider codeProvider = new CSharpCodeProvider();

        //Create Code Compile Unit
        CodeCompileUnit compileUnit = new CodeCompileUnit();

        //Create namespace
        CodeNamespace recipesNamespace = new CodeNamespace("Recipes");
        
        //Create namespace references
        recipesNamespace.Imports.Add(new CodeNamespaceImport("System"));

        //Create Round Class Declaration
        CodeTypeDeclaration round = new CodeTypeDeclaration();
        CodeComment roundComment = new CodeComment();
        roundComment.Text = "Round calculates the Area of a Circle, the Area of a Sphere, and the Volume of a Sphere";
        CodeCommentStatement roundCommentStatement = new CodeCommentStatement(roundComment);
        round.Comments.Add(roundCommentStatement);
        round.IsClass = true;
        round.Name = "Round";
        round.TypeAttributes = TypeAttributes.Public;

        //Create Class Member Field radius
        CodeMemberField radiusMember = new CodeMemberField(typeof(double),"radius");
        radiusMember.Attributes = MemberAttributes.Private;
        radiusMember.Comments.Add(new CodeCommentStatement("The radius used to calculate areas and volume."));
        round.Members.Add(radiusMember);

        //Create Round Class Constructor
        CodeConstructor roundConstructor = new CodeConstructor();
        roundConstructor.Attributes = MemberAttributes.Public;
        CodeParameterDeclarationExpression roundConParameter = new CodeParameterDeclarationExpression(typeof(double), "pRadius");
        roundConstructor.Parameters.Add(roundConParameter);
        roundConstructor.Statements.Add(new CodeSnippetStatement("this.radius = pRadius;"));
        round.Members.Add(roundConstructor);

        //Create Radius Property
        CodeMemberProperty radiusProperty = new CodeMemberProperty();
        radiusProperty.Attributes = MemberAttributes.Public;
        radiusProperty.Type = new CodeTypeReference(typeof(double));
        radiusProperty.Name = "Radius";
        radiusProperty.HasGet = true;
        radiusProperty.GetStatements.Add(new CodeSnippetExpression("return radius"));
        radiusProperty.HasSet = true;
        radiusProperty.SetStatements.Add(new CodeSnippetExpression("radius = value"));
        round.Members.Add(radiusProperty);

        //Create CircleArea Method
        CodeMemberMethod circleAreaMethod = new CodeMemberMethod();
        circleAreaMethod.Comments.Add(new CodeCommentStatement("Calculates the area of a circle."));
        circleAreaMethod.Name = "CircleArea";
        circleAreaMethod.ReturnType = new CodeTypeReference(typeof(double));
        circleAreaMethod.Attributes = MemberAttributes.Public;
        circleAreaMethod.Statements.Add(new CodeSnippetStatement("return Math.PI * radius * radius;"));
        round.Members.Add(circleAreaMethod);

        //Create SphereArea Method
        CodeMemberMethod sphereAreaMethod = new CodeMemberMethod();
        sphereAreaMethod.Comments.Add(new CodeCommentStatement("Calculates the area of a sphere."));
        sphereAreaMethod.Name = "SphereArea";
        sphereAreaMethod.ReturnType = new CodeTypeReference(typeof(double));
        sphereAreaMethod.Attributes = MemberAttributes.Public;
        sphereAreaMethod.Statements.Add(new CodeSnippetStatement("return 4 * CircleArea();"));
        round.Members.Add(sphereAreaMethod);

        //Create SphereVolume Method
        CodeMemberMethod sphereVolumeMethod = new CodeMemberMethod();
        sphereVolumeMethod.Comments.Add(new CodeCommentStatement("Calculates the volume of a sphere."));
        sphereVolumeMethod.Name = "SphereVolume";
        sphereVolumeMethod.ReturnType = new CodeTypeReference(typeof(double));
        sphereVolumeMethod.Attributes = MemberAttributes.Public;
        sphereVolumeMethod.Statements.Add(new CodeSnippetStatement("return SphereArea() * radius / 3;"));
        round.Members.Add(sphereVolumeMethod);

        //Add Round class to Recipe namespace
        recipesNamespace.Types.Add(round);

        //Add Namespace to Code Compile Unit
        compileUnit.Namespaces.Add(recipesNamespace);
        
        //Create ICodeCompiler
        ICodeCompiler compiler = codeProvider.CreateCompiler();

        //input parameters for the compiler
        CompilerParameters compilerParams = new CompilerParameters();
        string filepath = Path.GetDirectoryName(Request.PhysicalPath);
        compilerParams.OutputAssembly = filepath + "\\Round.dll";
        compilerParams.ReferencedAssemblies.Add("System.dll");
        compilerParams.GenerateExecutable = false; //dll not exe

        //Compile the assembly using the Compile Unit
        CompilerResults results = compiler.CompileAssemblyFromDom(compilerParams, compileUnit);

        StringBuilder sb = new StringBuilder();
        foreach (CompilerError error in results.Errors)
        {
          sb.Append(error.ErrorText);
          sb.Append("<br>");
        }

        foreach(string output in results.Output)
        {
          sb.Append(output);
          sb.Append("<br>");
        }

        return "Round Class compiled. " + sb.ToString();
      }
      catch(Exception ex)
      {
        return "Error: " + ex.Message;
      }
    }

    private void btnGenerate_Click(object sender, System.EventArgs e)
    {
      Result.Text = CreateCsCode();
    }
  </script>
    <title>Compiling a Class Using the CodeDOM</title>
  </head>
  <body>
    <form id="CreateCode" method="post" runat="server">
      <asp:Button id="btnGenerate" runat="server" Text="Create Code" OnClick="btnGenerate_Click"></asp:Button><br>
      <br>
      <asp:Label id="Result" runat="server"></asp:Label>
    </form>
  </body>
</html>
